"""Functions to manipulate and format date strings"""

import calendar
import os
from datetime import datetime, date
import re
from typing import Optional


def convert_date_string_to_text_string(
    date_str: str,
    date_format: Optional[str] = "%Y-%m-%d",
    include_day_of_month: Optional[bool] = True,
    abbreviate_month: Optional[bool] = True,
    include_year: Optional[bool] = True,
) -> str:
    """Converts date string in format passed in to text string, with format determined by
    parameters.
    Args:
        date_str (str): The date string to format.
        date_format (str, optional): The format of the "date" string. Defaults to "%Y-%m-%d".
        include_day_of_month (bool, optional): Whether to include the date.
        abbreviate_month (bool, optional): Whether to abbreviate the month eg. Mar over March.
            Defaults to True.
        include_year (bool, optional): Whether to include the year. Defaults to True.
    Returns:
        str: Formatted date string.
    """
    dt_obj = datetime.strptime(date_str, date_format)
    return convert_date_obj_to_text_string(
        dt_obj, include_day_of_month, abbreviate_month, include_year
    )


def convert_date_obj_to_text_string(
    dt_obj: datetime,
    include_day_of_month: Optional[bool] = True,
    abbreviate_month: Optional[bool] = True,
    include_year: Optional[bool] = True,
) -> str:
    """Converts date object to text string, with format determined by parameters. If month is June
    or July, abbreviate_month is set to False.
    Args:
        dt_obj (datetime): The datetime object to format, in form YYYY-MM-DD.
        include_day_of_month (bool, optional): Whether to include the date.
        abbreviate_month (bool, optional): Whether to abbreviate the month eg. Mar over March.
            Defaults to True.
        include_year (bool, optional): Whether to include the year. Defaults to True.
    Returns:
        str: Formatted date string.
    """
    if dt_obj.month in (6, 7):
        abbreviate_month = False
    if include_day_of_month:
        text_string_format = "%e"
        if abbreviate_month:
            text_string_format += " %b"
        else:
            text_string_format += " %B"
    else:
        text_string_format = ""
        if abbreviate_month:
            text_string_format += "%b"
        else:
            text_string_format += "%B"
    if include_year:
        text_string_format += " %Y"
    return dt_obj.strftime(text_string_format).strip()


def format_year_month_to_month_year(date_string: str):
    """
    Converts a date string from 'YYYY-MM' format to 'Month YYYY' format.
    Args:
    date_string (str): A date string in the format 'YYYY-MM'.
    Returns:
    str: The formatted date string in 'Month YYYY' format.
    """
    if not date_string:
        raise ValueError("No date_string provided")
    return datetime.strptime(date_string, "%Y-%m").strftime("%B %Y")


def convert_financial_year_to_date(
    financial_year_str: str, use_start_date: bool = True
) -> datetime:
    """Converts a financial year string to a datetime object.
    If `use_start_date` is True, the function will return April 1st of the
    start year of the financial year. If `use_start_date` is False,
    it will return March 31st of the end year of the financial year.

    Args:
        financial_year_str (str): Financial year string in format YYYY-YY
        use_start_date (bool, optional): Whether to use start or end date. Defaults to True.

    Returns:
        datetime.datetime: A datetime object corresponding to the requested date
                (either April 1st of the start year or March 31st of the end year).
    """
    start_year = int(financial_year_str[:4])
    end_year = start_year + 1
    if use_start_date:
        date_str = f"{start_year}-04-01"
    else:
        date_str = f"{end_year}-03-31"
    return datetime.strptime(date_str, "%Y-%m-%d")


def convert_calendar_year_to_first_january_date(date_str: str):
    """Converts a string in format YYYY to datetime object on January 1st"""
    date_str = f"{date_str}-01-01"
    return datetime.strptime(date_str, "%Y-%m-%d")


def convert_to_dd_mm_yyyy(date_str: str, includes_time: bool = False) -> str:
    """Converts a date string to a string in the format DD-MM-YYYY

    Args:
        date_str (str): Date string, either in format YYYY-MM-DD OR YYYY-DD-HH HH:MM:SS.MsMsMs
        includes_time (bool, optional): Whether the date_str contains time. Defaults to False.

    Returns:
        str: String in the format DD-MM-YYYY
    """
    datetime_format = "%Y-%m-%d %H:%M:%S.%f" if includes_time else "%Y-%m-%d"
    datetime_object = datetime.strptime(date_str, datetime_format)
    return datetime_object.strftime("%d-%m-%Y")


def convert_to_financial_year(date_str: str, date_format: str = "%d-%m-%Y") -> str:
    """Convert a date string to financial year format (e.g., '01-04-2023' to '2023-24')"""
    date_obj = datetime.strptime(date_str, date_format)
    year = date_obj.year
    if date_obj.month < 4:
        financial_year = f"{year-1}-{str(year)[-2:]}"
    else:
        financial_year = f"{year}-{str(year+1)[-2:]}"
    return financial_year


def convert_to_financial_year_ending(
    date_str: str, date_format: str = "%d-%m-%Y"
) -> str:
    """Convert a date string to financial year ending format (e.g., '01-04-2023' to '2024')"""
    date_obj = datetime.strptime(date_str, date_format)
    year = date_obj.year
    if date_obj.month < 4:
        return f"{year}"
    return f"{year + 1}"


def get_todays_date() -> str:
    """get a string of todays date fixing value for visual tests"""
    if os.environ.get("STAGE") and os.environ.get("STAGE") == "testing":
        return "2023-12-25"
    return str(date.today())


def get_todays_date_for_downloaded_csv() -> str:
    """get a string of todays date for use in download csv, fixing value for testing"""
    if os.environ.get("STAGE") and os.environ.get("STAGE") == "testing":
        return "25/12/2023"
    return str(datetime.today().strftime("%d/%m/%Y"))


def date_string_is_full_month_and_full_year_format(date_string: str):
    """checks if a date string is in the format $B $Y"""
    try:
        datetime.strptime(date_string, "%B %Y")
        return True
    except ValueError:
        return False


def convert_to_date_range(date_str: str) -> str:
    """Convert a date string in the format "YYYY-MM-DD", to a date range from the previous year to
    the current year for the month of the provided date. Abbreviates the month if it isn't June or
    July.

    Args:
        date_str (str): A date string in the format "YYYY-MM-DD".

    Returns:
        str: Date range in the format "MMM YYYY to MMM YYYY", where MMM is the abbreviated month
        name, if it isn't June or July. eg. Mar 2021 to Mar 2022
    """
    date_object = datetime.strptime(date_str, "%Y-%m-%d").date()

    if date_object.month in (6, 7):
        current_month = date_object.strftime("%B")
    else:
        current_month = date_object.strftime("%b")

    current_year = date_object.year

    previous_year = current_year - 1

    date_range = f"{current_month} {previous_year} to {current_month} {current_year}"

    return date_range


def shorten_month_range(date_str: str) -> str:
    """Shortens months when given a range in the format Month to Month YYYY.
        eg. January to February 2024 becomes Jan to Feb 2024.
    Args:
        date_str (str): String in format "Month to Month YYYY"
    Returns:
        str: String with abbreviated months
    """
    parts = date_str.split()
    start_month, end_month, year = parts[0], parts[2], parts[3]

    start_month_short = calendar.month_abbr[
        list(calendar.month_name).index(start_month)
    ]
    end_month_short = calendar.month_abbr[list(calendar.month_name).index(end_month)]

    return f"{start_month_short} to {end_month_short} {year}"


def date_string_is_month_to_month_year_range(date_str: str) -> bool:
    """Check if the input string is in the format "Month to Month Year".
        This function uses a regular expression to verify whether the provided
        string follows the specific format where the first and second words are
        full month names, followed by "to" and a four-digit year.
    Args:
        date_str (str): A string representing a date range in the format "Month to Month Year".
    Returns:
        bool: True if the string matches the format; False otherwise.
    """
    # Regex pattern to match "Month to Month Year"
    pattern = r"^(January|February|March|April|May|June|July|August|September|October|November|December) to (January|February|March|April|May|June|July|August|September|October|November|December) \d{4}$"  # pylint: disable=(line-too-long)
    return bool(re.match(pattern, date_str))


def convert_april_date_string_to_fye(date_str: str):
    """Converts an april date to the year in which that financial year ends"""
    year = str(int(date_str[:4]) + 1)
    return year


def get_year_from_date_object(date_obj):
    """Returns a year from a date object"""
    year = date_obj.year
    return str(year)


def replace_jun_jul_month_abbreviations(month_year_list: list[str]) -> list[str]:
    """Replace abbreviated Jun and Jul month names in a list of "Month Year" or "Month" strings with
    their full names. eg. Jun 2024 returns June 2024 and Jul returns July. Other parts of the
    strings remain unchanged.

    Args:
        month_year_list (list[str]): A list of strings containing abbreviated month names followed
            by a year or month alone (e.g., ["Jun 2023", "Jul 2024", "Jun"])

    Returns:
        list[str]: A list of strings with abbreviated Jun and Jul replaced by full month.
    """
    month_map = {"Jun": "June", "Jul": "July"}
    updated_list = [
        " ".join([month_map.get(part, part) for part in item.split()])
        for item in month_year_list
    ]
    return updated_list


def format_date_with_custom_months(date_object: datetime.date, include_day=True) -> str:
    """Format a date object into a custom string, in format "day month year", where month is
    abbreviated, unless month is June or July. eg. 10 Jan 2023, 14 June 2024.

    Args:
        date_object (datetime.date): A `datetime.date` object to format.
        include_day (bool): Boolean to determine whether to include day of month in output

    Returns:
        str: The formatted date string with custom month formatting.
    """
    if date_object.month in (6, 7):
        month = date_object.strftime("%B")
    else:
        month = date_object.strftime("%b")

    day = date_object.day
    year = date_object.year
    if include_day:

        return f"{day} {month} {year}"
    return f"{month} {year}"


def weeks_between_dates(date1: str, date2: str) -> float:
    """Calculate the unrounded number of weeks between two dates provided as strings in the format
    'YYYY-MM-DD'.

    Args:
        date1 (str): The first date in 'YYYY-MM-DD' format.
        date2 (str): The second date in 'YYYY-MM-DD' format.

    Returns:
        float: Number of weeks between the two dates.
    """

    date_format = "%Y-%m-%d"
    d1 = datetime.strptime(date1, date_format)
    d2 = datetime.strptime(date2, date_format)
    return abs((d2 - d1).days) / 7


def convert_date_to_financial_quarter(date_str: str):
    """Convert date string in the format yyyy-mm-dd to a financial quarter

    Args:
        date_str (str): yyyy-mm-dd

    Returns:
        integer with april-june being 1 and so on
    """
    date_obj = datetime.strptime(date_str, "%Y-%m-%d")
    if date_obj.month < 4:
        return 4
    if date_obj.month < 7:
        return 1
    if date_obj.month < 10:
        return 2
    return 3


def convert_financial_quarter_to_financial_quarter_text(quarter: int):
    """Convert quarter integer to financial quarter text including months

    Args:
        quarter (str): 1, 2, 3, 4 representing quarters with apr-june being 1

    Returns:
        integer with april-june being 1 and so on
    """
    quarter_map = {
        1: "Q1 (Apr-June)",
        2: "Q2 (July-Sep)",
        3: "Q3 (Oct-Dec)",
        4: "Q4 (Jan-Mar)",
    }
    return quarter_map[quarter]



"""download_csv_with_headers"""
import io
import polars as pl
from dash import dcc
from gov_uk_dashboards.lib.datetime_functions.datetime_functions import (
    get_todays_date_for_downloaded_csv,
)


def download_csv_with_headers(
    list_of_df_title_subtitle_dicts: list[dict[str, str]],
    name: str,
    sensitivity_label: str,
    additional_text: list[str] = None,
):  # pylint: disable=too-many-locals
    """Adds a header above multiple dataframes,
    separates them with blank rows, and downloads as CSV.

    Args:
        list_of_df_title_subtitle_dicts (list[dict[]]): List of dictionaries containing keys: "df",
        "title" and "subtitle"
        name (str): Filename for CSV.
        sensitivity_label (str): Sensitivity label. Str or None.
        additional_text (list[str]): Additional text to inlcude in headers after data downloaded.
            Str or None.
    """

    csv_buffer = io.StringIO()

    column_list = list(list_of_df_title_subtitle_dicts[0]["df"].columns)
    column_dict = {column_name: column_name for column_name in column_list}
    blank_dict = {
        f"{i}": None
        for i in range(
            _get_number_of_max_columns_from_all_dfs(list_of_df_title_subtitle_dicts)
            - len(column_list)
        )
    }  # range is missing columns in first df compared to max columns across all dfs

    subtitle = list_of_df_title_subtitle_dicts[0]["subtitle"]
    footnote = list_of_df_title_subtitle_dicts[0].get("footnote")
    header_data = [
        {column_list[0]: "Date downloaded: " + get_todays_date_for_downloaded_csv()},
        *(
            [{column_list[0]: text} for text in additional_text]
            + [{column_list[0]: None}]
            if additional_text is not None
            else []
        ),
        {column_list[0]: list_of_df_title_subtitle_dicts[0]["title"]},
        *(
            [{column_list[0]: subtitle}] if subtitle is not None else []
        ),  # Uses unpacking (*) to add the subtitle row if subtitle is not None. If subtitle is
        # None, it unpacks an empty list, effectively skipping the row.
        {column_list[0]: None},  # Blank row
        *([{column_list[0]: footnote}] if footnote is not None else []),
        {**column_dict, **blank_dict},
    ]

    if sensitivity_label:
        header_data = [{column_list[0]: sensitivity_label}] + header_data

    pl.DataFrame(header_data).write_csv(csv_buffer, include_header=False)
    for i, data in enumerate(list_of_df_title_subtitle_dicts):
        df = data["df"]
        title = data["title"]
        subtitle = data["subtitle"]
        footnote = data.get("footnote")
        if i > 0 and title is not None:
            column_dict = {column_name: column_name for column_name in list(df.columns)}
            header_data = [
                {column_list[0]: title},
                *(
                    [{column_list[0]: subtitle}] if subtitle is not None else []
                ),  # Uses unpacking (*) to add the subtitle row if subtitle is not None. If
                # subtitle is None, it unpacks an empty list, effectively skipping the row.
                {column_list[0]: None},  # Blank row
                *([{column_list[0]: footnote}] if footnote is not None else []),
            ]
            pl.DataFrame(header_data).write_csv(csv_buffer, include_header=False)
        df.write_csv(csv_buffer, include_header=i > 0)

        if i < len(list_of_df_title_subtitle_dicts) - 1:
            blank_row = pl.DataFrame({df.columns[0]: [None]})
            blank_row.write_csv(csv_buffer, include_header=False)

    csv_buffer.seek(0)
    csv_data = (
        "\ufeff" + csv_buffer.getvalue()
    )  # Adding \ufeff ensures the correct character encoding is detected for £

    return dcc.send_string(csv_data, f"{name}.csv")


def _get_number_of_max_columns_from_all_dfs(list_of_df_title_subtitle_dicts):
    max_columns = 0
    index_of_max_cols = -1

    for idx, dic in enumerate(list_of_df_title_subtitle_dicts):
        # Get the DataFrame
        df = dic["df"]

        # Get the number of columns
        num_columns = df.shape[1]

        # Update if this DataFrame has more columns
        if num_columns > max_columns:
            max_columns = num_columns
            index_of_max_cols = idx

    max_columns = len(
        list(list_of_df_title_subtitle_dicts[index_of_max_cols]["df"].columns)
    )

    return max_columns



"""Creates a Dash Graph component from a given Plotly figure"""

from typing import Any, Dict, Union
import plotly.graph_objects as Figure
from dash import dcc


def generate_dash_graph_from_figure(
    figure: Figure,
    graph_name: str,
    graph_style: Union[Dict[str, Any], None] = None,
    double_click_attribute: Union[str, bool] = False,
    class_name: str = "",
) -> dcc.Graph:
    """
    Creates a Dash Graph component from a given Plotly figure. This function allows for the
    customisation of the graph's appearance and behavior, and includes default pointer cursor,
    from "default-cursor-graph" class.
    Args:
    - figure: Plotly.graph_objects.Figure instance to be displayed within the Dash Graph component.
    - graph_name: A name for the id of the graph component.
    - graph_style: An optional dictionary specifying CSS styles to apply to the graph component.
    - double_click_attribute: Determines the action taken on double-clicking the graph.
                              Can be a boolean or a string specifying the mode.
    - className (str): A string containing one or more CSS class names to apply to the graph.
    Returns:
    - A dash.dcc.Graph component configured with the provided parameters and styles.
    """

    if not graph_style:
        graph_style = {}

    if "height" not in graph_style.keys():
        graph_style["height"] = "450px"

    figure.update_layout(dragmode=False)
    # pylint: disable=duplicate-code
    return dcc.Graph(
        id=f"{graph_name}-graph",
        responsive=True,
        figure=figure,
        style=graph_style,
        config={
            "displayModeBar": False,
            "doubleClick": double_click_attribute,
            "scrollZoom": False,
        },
        className="default-cursor-graph " + class_name,
    )



"""Leaflet choropleth map class"""

from typing import Optional
from dash_extensions.javascript import arrow_function, Namespace
import dash_leaflet as dl
import dash_leaflet.express as dlx
from dash import html
import polars as pl

from gov_uk_dashboards.components.helpers.display_chart_or_table_with_header import (
    display_chart_or_table_with_header,
)
from gov_uk_dashboards.formatting.number_formatting import (
    format_number_into_thousands_or_millions,
)


class LeafletChoroplethMap:
    """Class for  generating leaflet choropleth map charts.
    Note: Values in the numeric column should use 1 for the highest value, with larger numbers
    representing lower values.
    If color_scale_is_discrete is false, colour scale will be continuous, otherwise it will be
    discrete"""

    # pylint: disable=too-few-public-methods
    # pylint: disable=too-many-instance-attributes
    # pylint: disable=too-many-arguments
    # pylint: disable=too-many-positional-arguments

    def __init__(
        self,
        geojson: dict,
        df: pl.DataFrame,
        hover_text_columns: list[str],
        column_to_plot: str,
        legend_column: str,
        area_column: str,
        title: str,
        instance: int,
        subtitle: Optional[str] = None,
        enable_zoom: bool = True,
        download_chart_button_id: Optional[str] = None,
        download_data_button_id: Optional[str] = None,
        color_scale_is_discrete: bool = True,
        show_tile_layer: bool = False,
    ):
        self.geojson_data = geojson
        self.df = df
        self.hover_text_columns = hover_text_columns
        self.column_to_plot = column_to_plot
        self.legend_column = legend_column
        self.area_column = area_column
        self.title = title
        self.subtitle = subtitle
        self.enable_zoom = enable_zoom
        self.download_chart_button_id = download_chart_button_id
        self.download_data_button_id = download_data_button_id
        self.color_scale_is_discrete = color_scale_is_discrete
        self.show_tile_layer = show_tile_layer
        self._add_data_to_geojson()
        self.instance = instance

    def get_leaflet_choropleth_map(self):
        """Creates and returns leaflet choropleth map chart for display on application.

        Returns:
            dl.Map: A dash leaflet map chart.
        """
        disabled_zoom_controls = {
            "scrollWheelZoom": False,
            "dragging": False,
            "zoomControl": False,
            "doubleClickZoom": False,
            "touchZoom": False,
        }
        zoom_controls = {} if self.enable_zoom else disabled_zoom_controls
        choropleth_map = dl.Map(
            children=[
                dl.TileLayer() if self.show_tile_layer else None,
                self._get_colorbar(),
                self._get_colorbar_title(self.enable_zoom),
                self._get_dl_geojson(),
            ],
            center=[54.5, -2.5],  # Centered on the UK
            zoom=6.5,
            minZoom=6.5,
            maxZoom=10 if self.enable_zoom else 6.5,
            maxBounds=[[49.8, -10], [55.9, 1.8]],
            **zoom_controls,
            attributionControl=False,
            style={"width": "100%", "height": "800px", "background": "white"},
        )
        download_choropleth_map = dl.Map(
            children=[
                dl.TileLayer() if self.show_tile_layer else None,
                self._get_colorbar(),
                self._get_colorbar_title(),
                self._get_dl_geojson(),
            ],
            center=[54.5, -25.0],
            zoom=7.5,
            maxBounds=[[49.5, -30], [60, 2]],
            zoomControl=False,
            attributionControl=False,
            style={"width": "1200px", "height": "1200px", "background": "white"},
        )
        choropleth_map = display_chart_or_table_with_header(
            choropleth_map,
            self.title,
            self.subtitle,
            None,
            self.download_data_button_id,
            self.download_chart_button_id,
            None,
            self.instance,
        )
        download_choropleth_map_display = display_chart_or_table_with_header(
            download_choropleth_map,
            self.title,
            self.subtitle,
        )

        return [
            choropleth_map,
            html.Div(
                [download_choropleth_map_display],
                id=f"{self.download_chart_button_id}-hidden-map-container",
                style={
                    "position": "absolute",
                    "top": "-10000px",
                    "left": "-10000px",
                },  # hide off screen
            ),
        ]

    def _add_data_to_geojson(self):
        info_map = {
            row["Area_Code"]: {
                "value": row[self.column_to_plot],
                "area": row[self.area_column],
                **{col: row[col] for col in self.hover_text_columns},
            }
            for row in self.df.iter_rows(named=True)
        }

        for feature in self.geojson_data["features"]:
            region_code = feature["properties"]["geo_id"]
            info = info_map.get(region_code)
            if info:

                feature["properties"]["density"] = info["value"]
                feature["properties"]["area"] = info["area"]

                tooltip_parts = [f"<b>{info['area']}</b>"]
                if info["value"] is None:
                    tooltip_parts.append("<br>No data available")
                else:
                    for col in self.hover_text_columns:
                        tooltip_parts.append(f"<br>{col}: {info[col]}")

                feature["properties"]["tooltip"] = "".join(tooltip_parts)
            else:
                feature["properties"]["density"] = None
                feature["properties"]["area"] = "Unknown"
                feature["properties"]["tooltip"] = "No data available"

    def _get_dl_geojson(self):
        style_handle = self._get_style_handle()
        colorscale = self._get_colorscale()
        style = {
            "weight": 2,
            "opacity": 1,
            "color": "white",
            "fillOpacity": 0.7 if self.show_tile_layer else 1,
        }
        hover_style = arrow_function({"weight": 5, "color": "#666", "dashArray": ""})
        return dl.GeoJSON(
            data=self.geojson_data,
            id="geojson",
            zoomToBounds=True,
            zoomToBoundsOnClick=True,
            style=style_handle,
            hoverStyle=hover_style,
            hideout={
                "colorscale": colorscale,  # Use hex strings
                "style": style,
                "colorProp": "density",
                "min": self.df[self.column_to_plot].min(),
                "max": self.df[self.column_to_plot].max(),
            },
        )

    def _get_style_handle(self):
        ns = Namespace("myNamespace", "mapColorScaleFunctions")
        if self.color_scale_is_discrete:
            return ns("discreteColorScale")
        return ns("continuousColorScale")

    def _get_colorscale(self):
        if self.color_scale_is_discrete:
            discrete_colours = ["#217847", "#23BBBE", "#8CCE69", "#FFEA80"]
            if len(self.df[self.column_to_plot].unique()) == 3:
                discrete_colours.pop(1)
            return discrete_colours
        return ["#B0F2BC", "#257D98"]

    def _get_color_bar_categories(self):
        return (
            self.df.select([self.legend_column, self.column_to_plot])
            .unique()
            .sort(self.column_to_plot, descending=True)
            .select(self.legend_column)
            .to_series()
            .to_list()
        )

    def _get_colorbar(self):
        if self.color_scale_is_discrete:
            self._get_color_bar_categories()
            return dlx.categorical_colorbar(
                categories=self._get_color_bar_categories(),  # reversed order
                colorscale=self._get_colorscale()[::-1],
                width=30,
                height=200,
                position="topleft",
                style={
                    "padding": "6px",
                    "backgroundColor": "white",
                    "borderRadius": "4px",
                    "fontSize": "16px",
                },
            )
        min_value = self.df.select(pl.min(self.column_to_plot)).item()
        colorbar_min = min(min_value, 0)
        max_value = self.df.select(pl.max(self.column_to_plot)).item()
        mid_value = (colorbar_min + max_value) / 2
        quarter_value = (colorbar_min + max_value) / 4
        three_quarter_value = 3 * (colorbar_min + max_value) / 4
        tick_values = [
            colorbar_min,
            quarter_value,
            mid_value,
            three_quarter_value,
            max_value,
        ]
        tick_text = [
            format_number_into_thousands_or_millions(x) for x in tick_values
        ]  # Optional, for formatting

        return dl.Colorbar(
            colorscale=self._get_colorscale(),
            width=20,
            height=200,
            min=colorbar_min,
            max=max_value,
            position="topleft",
            style={
                "backgroundColor": "white",
                "padding": "5px",
                "borderRadius": "4px",
                "marginTop": "100px",
            },
            tickValues=tick_values,
            tickText=tick_text,  # Optional, makes labels look cleaner
        )

    def _get_colorbar_title(self, enable_zoom: bool = False):
        if self.color_scale_is_discrete:
            return None
        top = "70px" if enable_zoom is False else "140px"
        return html.Div(
            self.hover_text_columns[0],
            style={
                "position": "absolute",
                "top": top,  # Adjusted to place above the colorbar
                "left": "10px",  # Align with the left side of the colorbar
                "background": "white",
                "padding": "2px 6px",
                "borderRadius": "5px",
                "fontWeight": "bold",
                "fontSize": "14px",
                "zIndex": "999",  # Ensure it appears above map elements
            },
        )
